from django.views.generic import DateDetailView, YearArchiveView, MonthArchiveView, \
    DayArchiveView, ArchiveIndexView
from blog.models import Entry

class EntryArchiveIndexView(ArchiveIndexView):
    date_field = 'date_published'
    model = Entry
    queryset = Entry.objects.published()

class EntryYearArchiveView(YearArchiveView):
    date_field = 'date_published'
    model = Entry
    queryset = Entry.objects.published()

class EntryMonthArchiveView(MonthArchiveView):
    date_field = 'date_published'
    model = Entry
    month_format = '%m'
    queryset = Entry.objects.published()

class EntryDayArchiveView(DayArchiveView):
    date_field = 'date_published'
    model = Entry
    month_format = '%m'
    queryset = Entry.objects.published()

class EntryDateDetailView(DateDetailView):
    date_field = 'date_published'
    model = Entry
    month_format = '%m'
    queryset = Entry.objects.published()


from django.conf.urls.defaults import *
from blog.views import EntryArchiveIndexView, EntryYearArchiveView, \
    EntryMonthArchiveView, EntryDayArchiveView, EntryDateDetailView

urlpatterns = patterns('',
    url(r'^$', EntryArchiveIndexView.as_view(), name='entry_archive_index'),
    url(r'^(?P<year>\d+)/$', EntryYearArchiveView.as_view(),
        name='entry_year_archive'),
    url(r'^(?P<year>\d+)/(?P<month>\d+)/$', EntryMonthArchiveView.as_view(),
        name='entry_month_archive'),
    url(r'^(?P<year>\d+)/(?P<month>\d+)/(?P<day>\d+)/$', EntryDayArchiveView.as_view(),
        name='entry_day_archive'),
    url(r'^(?P<year>\d+)/(?P<month>\d+)/(?P<day>\d+)/(?P<slug>[\w-]+)/$', EntryDateDetailView.as_view(),
        name='entry_date_detail'),
)


from datetime import date, datetime, timedelta
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test.client import Client
from blog.models import Entry

class BlogTest(TestCase):
    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_user('test', 'test@test.com', 'test')

    def test_entry_archive_index(self):
        # No news - 404
        response = self.client.get(reverse('entry_archive_index'))
        self.assertEqual(response.status_code, 404)

        # Create a blog entry
        e = Entry.objects.create(headline='Test 1', content='Testing', status=1,
                                 date_published=datetime.now() - timedelta(seconds=10),
                                 author=self.user)
        response = self.client.get(reverse('entry_archive_index'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['object_list']), 1)
        assert e in response.context['object_list']

    def test_entry_year_archive(self):
        # No blog entries - 404
        response = self.client.get(reverse('entry_year_archive', kwargs={'year': datetime.now().year}))
        self.assertEqual(response.status_code, 404)

        # Create a blog entry
        now = datetime.now() - timedelta(seconds=10)
        Entry.objects.create(headline='Test 1', content='Test', status=1, date_published=now, author=self.user)
        response = self.client.get(reverse('entry_year_archive', kwargs={'year': now.year}))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['year'], str(now.year))
        self.assertEqual(len(response.context['date_list']), 1)
        self.assertEqual(response.context['date_list'][0], datetime(now.year, now.month, 1))

    def test_entry_month_archive(self):
        # No blog entries - 404
        now = datetime.now() - timedelta(seconds=10)
        response = self.client.get(reverse('entry_month_archive',
                                           kwargs={'year': now.year, 'month': now.month}))
        self.assertEqual(response.status_code, 404)

        # Create a blog entry
        e = Entry.objects.create(headline='Test 1', content='Test', status=1, date_published=now, author=self.user)
        response = self.client.get(reverse('entry_month_archive',
                                           kwargs={'year': now.year, 'month': now.month}))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['month'], date(now.year, now.month, 1))
        self.assertEqual(len(response.context['date_list']), 1)
        self.assertEqual(response.context['date_list'][0], datetime(now.year, now.month, now.day))

    def test_entry_day_archive(self):
        # No blog entries - 404
        now = datetime.now() - timedelta(seconds=10)
        response = self.client.get(reverse('entry_day_archive',
                                           kwargs={
                                               'year': now.year,
                                               'month': now.month,
                                               'day': now.day
                                           }))
        self.assertEqual(response.status_code, 404)

        # Create a blog entry
        e = Entry.objects.create(headline='Test 1', content='Test', status=1, date_published=now, author=self.user)
        response = self.client.get(reverse('entry_day_archive',
                                           kwargs={
                                               'year': now.year,
                                               'month': now.month,
                                               'day': now.day
                                           }))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['day'], date(now.year, now.month, now.day))
        self.assertEqual(len(response.context['object_list']), 1)
        self.assertEqual(response.context['object_list'][0], e)

    def test_entry_date_detail(self):
        # Blog entry not published - 404
        now = datetime.now() - timedelta(seconds=10)
        e = Entry.objects.create(headline='Test 1', content='Test', status=0, date_published=now, author=self.user)
        response = self.client.get(e.get_absolute_url())
        self.assertEqual(response.status_code, 404)

        # Publish this blog entry
        e.status = 1
        e.save()
        response = self.client.get(e.get_absolute_url())
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['object'], e)



from datetime import datetime
from autoslug import AutoSlugField
from django.db import models
from taggit.managers import TaggableManager

class EntryManager(models.Manager):
    def drafts(self):
        """
        Returns a queryset with drafts
        """
        return self.get_query_set().filter(status=0)

    def published(self):
        """
        Returns a queryset with published entries
        """
        return self.get_query_set().filter(status=1, date_published__lte=datetime.now())

    def recent(self, limit=2):
        """
        Returns a queryset with recent entries
        """
        return self.published()[:limit]

STATUS_CHOICES = (
    (0, 'Draft'),
    (1, 'Published'),
)

class Entry(models.Model):
    headline = models.CharField(max_length=100)
    slug = AutoSlugField(max_length=100, populate_from='headline', unique=True)
    intro = models.CharField(max_length=100, blank=True, help_text='A short introductory text')
    content = models.TextField()
    status = models.IntegerField(choices=STATUS_CHOICES, default=0)
    author = models.ForeignKey('auth.User', related_name='entries')

    date_created = models.DateTimeField(auto_now_add=True, blank=True, null=True, editable=False)
    date_changed = models.DateTimeField(auto_now=True, blank=True, null=True, editable=False)
    date_published = models.DateTimeField('Publication date', default=datetime.now,
                                          help_text='Use future date and time for delayed publications')

    objects = EntryManager()
    tags = TaggableManager()

    class Meta:
        ordering = ('-date_published',)
        verbose_name_plural = 'Entries'

    def __unicode__(self):
        return self.headline

    @models.permalink
    def get_absolute_url(self):
        return ('entry_date_detail', (), {
            'year': self.date_published.year,
            'month': self.date_published.month,
            'day': self.date_published.day,
            'slug': self.slug
        })



from django.contrib.sitemaps import Sitemap
from blog.models import Entry

class EntrySitemap(Sitemap):
    changefreq = 'never'
    priority = 0.1

    def items(self):
        return Entry.objects.published()

    def lastmod(self, obj):
        return obj.date_published


from django.contrib.sites.models import Site
from django.contrib.syndication.feeds import Feed
from django.utils.feedgenerator import Rss201rev2Feed
from blog.models import Entry

class EntryLatest(Feed):
    feed_type = Rss201rev2Feed
    title_template = None
    description_template = None

    title = 'Blog'
    link =  '/blog/'
    language = 'en'
    description = ''

    def feed_copyright(self):
        site = Site.objects.get_current()
        return u'(c) %s' % site.name

    def items(self):
        return Entry.objects.published()

    def item_pubdate(self, item):
        return item.date_published

    def item_description(self, item):
        return item.content


from setuptools import setup, find_packages

setup(
    name='django-blog',
    version='0.1',
    author='Michael Samoylov',
    author_email='mike@djangoware.com',
    url='http://github.com/djangoware/django-blog',
    description = 'A simple blog app',
    packages=find_packages(),
    zip_safe=False,
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Topic :: Software Development'
    ],
)



from django.contrib import admin
from blog.models import Entry

class EntryAdmin(admin.ModelAdmin):
    actions = None
    list_display = ('headline', 'date_published', 'status')
    list_display_links = ('headline',)
    list_filter = ('status',)
    ordering = ('-date_published',)
    search_fields = ('headline',)

    class Media:
        js = (
            'scripts/jquery/jquery-1.4.2.min.js',
            'tiny_mce/tiny_mce.js',
            'tiny_mce/tinymce_setup.js',
            )

admin.site.register(Entry, EntryAdmin)


# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Entry'
        db.create_table('blog_entry', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('headline', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('slug', self.gf('autoslug.fields.AutoSlugField')(unique=True, max_length=100, populate_from=None, unique_with=(), db_index=True)),
            ('intro', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
            ('content', self.gf('django.db.models.fields.TextField')()),
            ('status', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('author', self.gf('django.db.models.fields.related.ForeignKey')(related_name='entries', to=orm['auth.User'])),
            ('date_created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, null=True, blank=True)),
            ('date_changed', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, null=True, blank=True)),
            ('date_published', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
        ))
        db.send_create_signal('blog', ['Entry'])


    def backwards(self, orm):
        
        # Deleting model 'Entry'
        db.delete_table('blog_entry')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'blog.entry': {
            'Meta': {'ordering': "('-date_published',)", 'object_name': 'Entry'},
            'author': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'entries'", 'to': "orm['auth.User']"}),
            'content': ('django.db.models.fields.TextField', [], {}),
            'date_changed': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'date_published': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'headline': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'intro': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique': 'True', 'max_length': '100', 'populate_from': 'None', 'unique_with': '()', 'db_index': 'True'}),
            'status': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'taggit.tag': {
            'Meta': {'object_name': 'Tag'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '100', 'db_index': 'True'})
        },
        'taggit.taggeditem': {
            'Meta': {'object_name': 'TaggedItem'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'taggit_taggeditem_tagged_items'", 'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.IntegerField', [], {'db_index': 'True'}),
            'tag': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'taggit_taggeditem_items'", 'to': "orm['taggit.Tag']"})
        }
    }

    complete_apps = ['blog']
